package com.compdfkit.demo.utils.threadpools;

import android.app.Activity;
import android.content.Context;
import android.os.AsyncTask;

import java.lang.ref.WeakReference;

/**
 * @classname：SimpleBackgroundTask
 * @date：27/7/18 10:53
 * @description： Async异步线程处理, 主要解决对leak的处理
 */
public abstract class SimpleBackgroundTask<T> extends AsyncTask<Void, Void, T> {
    WeakReference<Context> weakActivity;

    public SimpleBackgroundTask(Context context) {
        weakActivity = new WeakReference<>(context);
    }

    @Override
    protected final T doInBackground(Void... voids) {
        return onRun();
    }

    @Override
    protected void onPostExecute(T t) {
        Activity activity = (Activity) weakActivity.get();
        if ((null == activity) || activity.isFinishing() || activity.isDestroyed()) {
            cancel(true);
            return;
        } else {
            onSuccess(t);
            cancel(true);
        }
    }

    public SimpleBackgroundTask<T> execute() {
        executeOnExecutor(ThreadPoolUtils.getInstance().poolExecutor);
        return this;
    }

    abstract protected T onRun();

    abstract protected void onSuccess(T result);
}
